@@ -953,7 +953,7 @@ static struct cgroup_process_info *lxc_cgroupfs_create(const char *name, const c
 				current_entire_path = NULL;
 				goto cleanup_name_on_this_level;
 			} else if (r < 0 && errno != EEXIST) {
-				SYSERROR("Could not create cgroup %s", current_entire_path);
+				SYSERROR("Could not create cgroup '%s' in '%s'.", current_entire_path, info_ptr->designated_mount_point->mount_point);
 				goto cleanup_from_error;
 			} else if (r == 0) {
 				/* successfully created */
@@ -961,7 +961,7 @@ static struct cgroup_process_info *lxc_cgroupfs_create(const char *name, const c
 				if (r < 0)
 					goto cleanup_from_error;
 				if (!init_cpuset_if_needed(info_ptr->designated_mount_point, current_entire_path)) {
-					ERROR("Failed to initialize cpuset in new '%s'.", current_entire_path);
+					ERROR("Failed to initialize cpuset for '%s' in '%s'.", current_entire_path, info_ptr->designated_mount_point->mount_point);
 					goto cleanup_from_error;
 				}
 				info_ptr->created_paths[info_ptr->created_paths_count++] = current_entire_path;
@@ -1002,7 +1002,7 @@ static struct cgroup_process_info *lxc_cgroupfs_create(const char *name, const c
 		continue;
 
 	cleanup_from_error:
-		/* called if an error occured in the loop, so we
+		/* called if an error occurred in the loop, so we
 		 * do some additional cleanup here
 		 */
 		saved_errno = errno;
@@ -1220,6 +1220,7 @@ static char *lxc_cgroup_get_hierarchy_path_data(const char *subsystem, struct cg
 	info = find_info_for_subsystem(info, subsystem);
 	if (!info)
 		return NULL;
+	prune_init_scope(info->cgroup_path);
 	return info->cgroup_path;
 }
 
@@ -1280,7 +1281,7 @@ static int lxc_cgroup_set_data(const char *filename, const char *value, struct c
 
 	subsystem = alloca(strlen(filename) + 1);
 	strcpy(subsystem, filename);
-	if ((p = index(subsystem, '.')) != NULL)
+	if ((p = strchr(subsystem, '.')) != NULL)
 		*p = '\0';
 
 	path = lxc_cgroup_get_hierarchy_abs_path_data(subsystem, d);
@@ -1298,7 +1299,7 @@ static int lxc_cgroupfs_set(const char *filename, const char *value, const char
 
 	subsystem = alloca(strlen(filename) + 1);
 	strcpy(subsystem, filename);
-	if ((p = index(subsystem, '.')) != NULL)
+	if ((p = strchr(subsystem, '.')) != NULL)
 		*p = '\0';
 
 	path = lxc_cgroup_get_hierarchy_abs_path(subsystem, name, lxcpath);
@@ -1316,7 +1317,7 @@ static int lxc_cgroupfs_get(const char *filename, char *value, size_t len, const
 
 	subsystem = alloca(strlen(filename) + 1);
 	strcpy(subsystem, filename);
-	if ((p = index(subsystem, '.')) != NULL)
+	if ((p = strchr(subsystem, '.')) != NULL)
 		*p = '\0';
 
 	path = lxc_cgroup_get_hierarchy_abs_path(subsystem, name, lxcpath);
@@ -1344,6 +1345,15 @@ static bool cgroupfs_mount_cgroup(void *hdata, const char *root, int type)
 		return false;
 	base_info = cgfs_d->info;
 
+	/* If we get passed the _NOSPEC types, we default to _MIXED, since we don't
+	 * have access to the lxc_conf object at this point. It really should be up
+	 * to the caller to fix this, but this doesn't really hurt.
+	 */
+	if (type == LXC_AUTO_CGROUP_FULL_NOSPEC)
+		type = LXC_AUTO_CGROUP_FULL_MIXED;
+	else if (type == LXC_AUTO_CGROUP_NOSPEC)
+		type = LXC_AUTO_CGROUP_MIXED;
+
 	if (type < LXC_AUTO_CGROUP_RO || type > LXC_AUTO_CGROUP_FULL_MIXED) {
 		ERROR("could not mount cgroups into container: invalid type specified internally");
 		errno = EINVAL;
@@ -1354,7 +1364,10 @@ static bool cgroupfs_mount_cgroup(void *hdata, const char *root, int type)
 	if (!path)
 		return false;
 	snprintf(path, bufsz, "%s/sys/fs/cgroup", root);
-	r = mount("cgroup_root", path, "tmpfs", MS_NOSUID|MS_NODEV|MS_NOEXEC|MS_RELATIME, "size=10240k,mode=755");
+	r = safe_mount("cgroup_root", path, "tmpfs",
+			MS_NOSUID|MS_NODEV|MS_NOEXEC|MS_RELATIME,
+			"size=10240k,mode=755",
+			root);
 	if (r < 0) {
 		SYSERROR("could not mount tmpfs to /sys/fs/cgroup in the container");
 		return false;
@@ -1442,6 +1455,24 @@ static bool cgroupfs_mount_cgroup(void *hdata, const char *root, int type)
 				goto out_error;
 			}
 
+			/* for read-only and mixed cases, we have to bind-mount the tmpfs directory
+			 * that points to the hierarchy itself (i.e. /sys/fs/cgroup/cpu etc.) onto
+			 * itself and then bind-mount it read-only, since we keep the tmpfs itself
+			 * read-write (see comment below)
+			 */
+			if (type == LXC_AUTO_CGROUP_MIXED || type == LXC_AUTO_CGROUP_RO) {
+				r = mount(abs_path, abs_path, NULL, MS_BIND, NULL);
+				if (r < 0) {
+					SYSERROR("error bind-mounting %s onto itself", abs_path);
+					goto out_error;
+				}
+				r = mount(NULL, abs_path, NULL, MS_REMOUNT|MS_BIND|MS_RDONLY, NULL);
+				if (r < 0) {
+					SYSERROR("error re-mounting %s readonly", abs_path);
+					goto out_error;
+				}
+			}
+
 			free(abs_path);
 			abs_path = NULL;
 
@@ -1487,13 +1518,21 @@ static bool cgroupfs_mount_cgroup(void *hdata, const char *root, int type)
 		parts = NULL;
 	}
 
-	/* try to remount the tmpfs readonly, since the container shouldn't
-	 * change anything (this will also make sure that trying to create
-	 * new cgroups outside the allowed area fails with an error instead
-	 * of simply causing this to create directories in the tmpfs itself)
+	/* We used to remount the entire tmpfs readonly if any :ro or
+	 * :mixed mode was specified. However, Ubuntu's mountall has the
+	 * unfortunate behavior to block bootup if /sys/fs/cgroup is
+	 * mounted read-only and cannot be remounted read-write.
+	 * (mountall reads /lib/init/fstab and tries to (re-)mount all of
+	 * these if they are not already mounted with the right options;
+	 * it contains an entry for /sys/fs/cgroup. In case it can't do
+	 * that, it prompts for the user to either manually fix it or
+	 * boot anyway. But without user input, booting of the container
+	 * hangs.)
+	 *
+	 * Instead of remounting the entire tmpfs readonly, we only
+	 * remount the paths readonly that are part of the cgroup
+	 * hierarchy.
 	 */
-	if (type != LXC_AUTO_CGROUP_RW && type != LXC_AUTO_CGROUP_FULL_RW)
-		mount(NULL, path, NULL, MS_REMOUNT|MS_RDONLY, NULL);
 
 	free(path);
 
@@ -1851,14 +1890,19 @@ static int do_cgroup_set(const char *cgroup_path, const char *sub_filename,
 static int do_setup_cgroup_limits(struct cgfs_data *d,
 			   struct lxc_list *cgroup_settings, bool do_devices)
 {
-	struct lxc_list *iterator;
+	struct lxc_list *iterator, *sorted_cgroup_settings, *next;
 	struct lxc_cgroup *cg;
 	int ret = -1;
 
 	if (lxc_list_empty(cgroup_settings))
 		return 0;
 
-	lxc_list_for_each(iterator, cgroup_settings) {
+	sorted_cgroup_settings = sort_cgroup_settings(cgroup_settings);
+	if (!sorted_cgroup_settings) {
+		return -1;
+	}
+
+	lxc_list_for_each(iterator, sorted_cgroup_settings) {
 		cg = iterator->elem;
 
 		if (do_devices == !strncmp("devices", cg->subsystem, 7)) {
@@ -1881,6 +1925,11 @@ static int do_setup_cgroup_limits(struct cgfs_data *d,
 	ret = 0;
 	INFO("cgroup has been setup");
 out:
+	lxc_list_for_each_safe(iterator, sorted_cgroup_settings, next) {
+		lxc_list_del(iterator);
+		free(iterator);
+	}
+	free(sorted_cgroup_settings);
 	return ret;
 }
 
@@ -2152,8 +2201,7 @@ static bool do_init_cpuset_file(struct cgroup_mount_point *mp,
 		SYSERROR("failed writing %s", childfile);
 
 out:
-	if (parentfile)
-		free(parentfile);
+	free(parentfile);
 	free(childfile);
 	return ok;
 }
@@ -2191,15 +2239,7 @@ static void *cgfs_init(const char *name)
 	if (!d->name)
 		goto err1;
 
-	/* if we are running as root, use system cgroup pattern, otherwise
-	 * just create a cgroup under the current one. But also fall back to
-	 * that if for some reason reading the configuration fails and no
-	 * default value is available
-	 */
-	if (geteuid() == 0)
-		d->cgroup_pattern = lxc_global_config_value("lxc.cgroup.pattern");
-	if (!d->cgroup_pattern)
-		d->cgroup_pattern = "%n";
+	d->cgroup_pattern = lxc_global_config_value("lxc.cgroup.pattern");
 
 	d->meta = lxc_cgroup_load_meta();
 	if (!d->meta) {
@@ -2221,12 +2261,9 @@ static void cgfs_destroy(void *hdata)
 
 	if (!d)
 		return;
-	if (d->name)
-		free(d->name);
-	if (d->info)
-		lxc_cgroup_process_info_free_and_remove(d->info);
-	if (d->meta)
-		lxc_cgroup_put_meta(d->meta);
+	free(d->name);
+	lxc_cgroup_process_info_free_and_remove(d->info);
+	lxc_cgroup_put_meta(d->meta);
 	free(d);
 }
 
