@@ -39,6 +39,7 @@
 #include <sys/mount.h>
 #include <netinet/in.h>
 #include <net/if.h>
+#include <poll.h>
 
 #include "error.h"
 #include "commands.h"
@@ -51,6 +52,10 @@
 #include "start.h"
 #include "state.h"
 
+#define CGM_SUPPORTS_GET_ABS 3
+#define CGM_SUPPORTS_NAMED 4
+#define CGM_SUPPORTS_MULT_CONTROLLERS 10
+
 #ifdef HAVE_CGMANAGER
 lxc_log_define(lxc_cgmanager, lxc);
 
@@ -107,21 +112,23 @@ static void process_lock_setup_atfork(void)
 #endif
 
 static NihDBusProxy *cgroup_manager = NULL;
+static int32_t api_version;
 
 static struct cgroup_ops cgmanager_ops;
 static int nr_subsystems;
-static char **subsystems;
+static char **subsystems, **subsystems_inone;
 static bool dbus_threads_initialized = false;
+static void cull_user_controllers(void);
 
 static void cgm_dbus_disconnect(void)
 {
-       if (cgroup_manager) {
-	       dbus_connection_flush(cgroup_manager->connection);
-	       dbus_connection_close(cgroup_manager->connection);
-               nih_free(cgroup_manager);
-       }
-       cgroup_manager = NULL;
-       cgm_unlock();
+	if (cgroup_manager) {
+		dbus_connection_flush(cgroup_manager->connection);
+		dbus_connection_close(cgroup_manager->connection);
+		nih_free(cgroup_manager);
+	}
+	cgroup_manager = NULL;
+	cgm_unlock();
 }
 
 #define CGMANAGER_DBUS_SOCK "unix:path=/sys/fs/cgroup/cgmanager/sock"
@@ -147,17 +154,6 @@ static bool cgm_dbus_connect(void)
 		cgm_unlock();
 		return false;
 	}
-	if (nih_dbus_setup(connection, NULL) < 0) {
-		NihError *nerr;
-		nerr = nih_error_get();
-		DEBUG("Unable to open cgmanager connection at %s: %s", CGMANAGER_DBUS_SOCK,
-			nerr->message);
-		nih_free(nerr);
-		dbus_error_free(&dbus_error);
-		dbus_connection_unref(connection);
-		cgm_unlock();
-		return false;
-	}
 	dbus_connection_set_exit_on_disconnect(connection, FALSE);
 	dbus_error_free(&dbus_error);
 	cgroup_manager = nih_dbus_proxy_new(NULL, connection,
@@ -173,18 +169,92 @@ static bool cgm_dbus_connect(void)
 		return false;
 	}
 
-	// force fd passing negotiation
-	if (cgmanager_ping_sync(NULL, cgroup_manager, 0) != 0) {
+	// get the api version
+	if (cgmanager_get_api_version_sync(NULL, cgroup_manager, &api_version) != 0) {
 		NihError *nerr;
 		nerr = nih_error_get();
-		ERROR("Error pinging cgroup manager: %s", nerr->message);
+		ERROR("Error cgroup manager api version: %s", nerr->message);
 		nih_free(nerr);
 		cgm_dbus_disconnect();
 		return false;
 	}
+	if (api_version < CGM_SUPPORTS_NAMED)
+		cull_user_controllers();
 	return true;
 }
 
+static bool cgm_supports_multiple_controllers;
+/*
+ * if cgm_all_controllers_same is true, then cgm_supports_multiple_controllers
+ * is true
+ */
+static bool cgm_all_controllers_same;
+
+/*
+ * Check whether we can use "all" when talking to cgmanager.
+ * We check two things:
+ * 1. whether cgmanager is new enough to support this.
+ * 2. whether the task we are interested in is in the same
+ *    cgroup for all controllers.
+ * In cgm_init (before an lxc-start) we care about our own
+ * cgroup.  In cgm_attach, we care about the target task's
+ * cgroup.
+ */
+static void check_supports_multiple_controllers(pid_t pid)
+{
+	FILE *f;
+	char *line = NULL, *prevpath = NULL;
+	size_t sz = 0;
+	char path[100];
+
+	cgm_supports_multiple_controllers = false;
+	cgm_all_controllers_same = false;
+
+	if (api_version < CGM_SUPPORTS_MULT_CONTROLLERS) {
+		cgm_supports_multiple_controllers = false;
+		return;
+	}
+
+	cgm_supports_multiple_controllers = true;
+
+	if (pid == -1)
+		sprintf(path, "/proc/self/cgroup");
+	else
+		sprintf(path, "/proc/%d/cgroup", pid);
+	f = fopen(path, "r");
+	if (!f)
+		return;
+
+	cgm_all_controllers_same = true;
+
+	while (getline(&line, &sz, f) != -1) {
+		/* file format: hierarchy:subsystems:group */
+		char *colon;
+		if (!line[0])
+			continue;
+
+		colon = strchr(line, ':');
+		if (!colon)
+			continue;
+		colon = strchr(colon+1, ':');
+		if (!colon)
+			continue;
+		colon++;
+		if (!prevpath) {
+			prevpath = alloca(strlen(colon)+1);
+			strcpy(prevpath, colon);
+			continue;
+		}
+		if (strcmp(prevpath, colon) != 0) {
+			cgm_all_controllers_same = false;
+			break;
+		}
+	}
+
+	fclose(f);
+	free(line);
+}
+
 static int send_creds(int sock, int rpid, int ruid, int rgid)
 {
 	struct msghdr msg = { 0 };
@@ -246,15 +316,19 @@ static bool lxc_cgmanager_escape(void)
 {
 	bool ret = true;
 	pid_t me = getpid();
+	char **slist = subsystems;
 	int i;
 
-	for (i = 0; i < nr_subsystems; i++) {
+	if (cgm_all_controllers_same)
+		slist = subsystems_inone;
+
+	for (i = 0; slist[i]; i++) {
 		if (cgmanager_move_pid_abs_sync(NULL, cgroup_manager,
-					subsystems[i], "/", me) != 0) {
+					slist[i], "/", me) != 0) {
 			NihError *nerr;
 			nerr = nih_error_get();
 			ERROR("call to cgmanager_move_pid_abs_sync(%s) failed: %s",
-					subsystems[i], nerr->message);
+					slist[i], nerr->message);
 			nih_free(nerr);
 			ret = false;
 			break;
@@ -265,18 +339,16 @@ static bool lxc_cgmanager_escape(void)
 }
 
 struct chown_data {
-	const char *controller;
 	const char *cgroup_path;
 	uid_t origuid;
 };
 
 static int do_chown_cgroup(const char *controller, const char *cgroup_path,
-		uid_t origuid)
+		uid_t newuid)
 {
 	int sv[2] = {-1, -1}, optval = 1, ret = -1;
 	char buf[1];
-
-	uid_t caller_nsuid = get_ns_uid(origuid);
+	struct pollfd fds;
 
 	if (socketpair(AF_UNIX, SOCK_DGRAM, 0, sv) < 0) {
 		SYSERROR("Error creating socketpair");
@@ -300,10 +372,10 @@ static int do_chown_cgroup(const char *controller, const char *cgroup_path,
 	}
 	/* now send credentials */
 
-	fd_set rfds;
-	FD_ZERO(&rfds);
-	FD_SET(sv[0], &rfds);
-	if (select(sv[0]+1, &rfds, NULL, NULL, NULL) < 0) {
+	fds.fd = sv[0];
+	fds.events = POLLIN;
+	fds.revents = 0;
+	if (poll(&fds, 1, -1) <= 0) {
 		ERROR("Error getting go-ahead from server: %s", strerror(errno));
 		goto out;
 	}
@@ -315,9 +387,10 @@ static int do_chown_cgroup(const char *controller, const char *cgroup_path,
 		SYSERROR("%s: Error sending pid over SCM_CREDENTIAL", __func__);
 		goto out;
 	}
-	FD_ZERO(&rfds);
-	FD_SET(sv[0], &rfds);
-	if (select(sv[0]+1, &rfds, NULL, NULL, NULL) < 0) {
+	fds.fd = sv[0];
+	fds.events = POLLIN;
+	fds.revents = 0;
+	if (poll(&fds, 1, -1) <= 0) {
 		ERROR("Error getting go-ahead from server: %s", strerror(errno));
 		goto out;
 	}
@@ -325,13 +398,14 @@ static int do_chown_cgroup(const char *controller, const char *cgroup_path,
 		ERROR("Error getting reply from server over socketpair");
 		goto out;
 	}
-	if (send_creds(sv[0], getpid(), caller_nsuid, 0)) {
+	if (send_creds(sv[0], getpid(), newuid, 0)) {
 		SYSERROR("%s: Error sending pid over SCM_CREDENTIAL", __func__);
 		goto out;
 	}
-	FD_ZERO(&rfds);
-	FD_SET(sv[0], &rfds);
-	if (select(sv[0]+1, &rfds, NULL, NULL, NULL) < 0) {
+	fds.fd = sv[0];
+	fds.events = POLLIN;
+	fds.revents = 0;
+	if (poll(&fds, 1, -1) <= 0) {
 		ERROR("Error getting go-ahead from server: %s", strerror(errno));
 		goto out;
 	}
@@ -347,6 +421,9 @@ out:
 static int chown_cgroup_wrapper(void *data)
 {
 	struct chown_data *arg = data;
+	char **slist = subsystems;
+	int i, ret = -1;
+	uid_t destuid;
 
 	if (setresgid(0,0,0) < 0)
 		SYSERROR("Failed to setgid to 0");
@@ -354,7 +431,27 @@ static int chown_cgroup_wrapper(void *data)
 		SYSERROR("Failed to setuid to 0");
 	if (setgroups(0, NULL) < 0)
 		SYSERROR("Failed to clear groups");
-	return do_chown_cgroup(arg->controller, arg->cgroup_path, arg->origuid);
+	cgm_dbus_disconnect();
+	if (!cgm_dbus_connect()) {
+		ERROR("Error connecting to cgroup manager");
+		return -1;
+	}
+	destuid = get_ns_uid(arg->origuid);
+
+	if (cgm_supports_multiple_controllers)
+		slist = subsystems_inone;
+
+	for (i = 0; slist[i]; i++) {
+		if (do_chown_cgroup(slist[i], arg->cgroup_path, destuid) < 0) {
+			ERROR("Failed to chown %s:%s to container root",
+				slist[i], arg->cgroup_path);
+			goto fail;
+		}
+	}
+	ret = 0;
+fail:
+	cgm_dbus_disconnect();
+	return ret;
 }
 
 /* Internal helper.  Must be called with the cgmanager dbus socket open */
@@ -373,31 +470,43 @@ static bool lxc_cgmanager_chmod(const char *controller,
 }
 
 /* Internal helper.  Must be called with the cgmanager dbus socket open */
-static bool chown_cgroup(const char *controller, const char *cgroup_path,
-			struct lxc_conf *conf)
+static bool chown_cgroup(const char *cgroup_path, struct lxc_conf *conf)
 {
 	struct chown_data data;
+	char **slist = subsystems;
+	int i;
 
 	if (lxc_list_empty(&conf->id_map))
 		/* If there's no mapping then we don't need to chown */
 		return true;
 
-	data.controller = controller;
 	data.cgroup_path = cgroup_path;
 	data.origuid = geteuid();
 
+	/* Unpriv users can't chown it themselves, so chown from
+	 * a child namespace mapping both our own and the target uid
+	 */
 	if (userns_exec_1(conf, chown_cgroup_wrapper, &data) < 0) {
 		ERROR("Error requesting cgroup chown in new namespace");
 		return false;
 	}
 
-	/* now chmod 775 the directory else the container cannot create cgroups */
-	if (!lxc_cgmanager_chmod(controller, cgroup_path, "", 0775))
-		return false;
-	if (!lxc_cgmanager_chmod(controller, cgroup_path, "tasks", 0775))
-		return false;
-	if (!lxc_cgmanager_chmod(controller, cgroup_path, "cgroup.procs", 0775))
-		return false;
+	/*
+	 * Now chmod 775 the directory else the container cannot create cgroups.
+	 * This can't be done in the child namespace because it only group-owns
+	 * the cgroup
+	 */
+	if (cgm_supports_multiple_controllers)
+		slist = subsystems_inone;
+
+	for (i = 0; slist[i]; i++) {
+		if (!lxc_cgmanager_chmod(slist[i], cgroup_path, "", 0775))
+			return false;
+		if (!lxc_cgmanager_chmod(slist[i], cgroup_path, "tasks", 0775))
+			return false;
+		if (!lxc_cgmanager_chmod(slist[i], cgroup_path, "cgroup.procs", 0775))
+			return false;
+	}
 
 	return true;
 }
@@ -427,6 +536,9 @@ static void *cgm_init(const char *name)
 		ERROR("Error connecting to cgroup manager");
 		return NULL;
 	}
+
+	check_supports_multiple_controllers(-1);
+
 	d = malloc(sizeof(*d));
 	if (!d) {
 		cgm_dbus_disconnect();
@@ -440,15 +552,8 @@ static void *cgm_init(const char *name)
 		goto err1;
 	}
 
-	/* if we are running as root, use system cgroup pattern, otherwise
-	 * just create a cgroup under the current one. But also fall back to
-	 * that if for some reason reading the configuration fails and no
-	 * default value is available
-	 */
-	if (geteuid() == 0)
-		d->cgroup_pattern = lxc_global_config_value("lxc.cgroup.pattern");
-	if (!d->cgroup_pattern)
-		d->cgroup_pattern = "%n";
+	d->cgroup_pattern = lxc_global_config_value("lxc.cgroup.pattern");
+
 	// cgm_create immediately gets called so keep the connection open
 	return d;
 
@@ -461,6 +566,7 @@ err1:
 static void cgm_destroy(void *hdata)
 {
 	struct cgm_data *d = hdata;
+	char **slist = subsystems;
 	int i;
 
 	if (!d || !d->cgroup_path)
@@ -469,12 +575,14 @@ static void cgm_destroy(void *hdata)
 		ERROR("Error connecting to cgroup manager");
 		return;
 	}
-	for (i = 0; i < nr_subsystems; i++)
-		cgm_remove_cgroup(subsystems[i], d->cgroup_path);
+
+	if (cgm_supports_multiple_controllers)
+		slist = subsystems_inone;
+	for (i = 0; slist[i]; i++)
+		cgm_remove_cgroup(slist[i], d->cgroup_path);
 
 	free(d->name);
-	if (d->cgroup_path)
-		free(d->cgroup_path);
+	free(d->cgroup_path);
 	free(d);
 	cgm_dbus_disconnect();
 }
@@ -486,13 +594,18 @@ static void cgm_destroy(void *hdata)
 static inline void cleanup_cgroups(char *path)
 {
 	int i;
-	for (i = 0; i < nr_subsystems; i++)
-		cgm_remove_cgroup(subsystems[i], path);
+	char **slist = subsystems;
+
+	if (cgm_supports_multiple_controllers)
+		slist = subsystems_inone;
+	for (i = 0; slist[i]; i++)
+		cgm_remove_cgroup(slist[i], path);
 }
 
 static inline bool cgm_create(void *hdata)
 {
 	struct cgm_data *d = hdata;
+	char **slist = subsystems;
 	int i, index=0, baselen, ret;
 	int32_t existed;
 	char result[MAXPATHLEN], *tmp, *cgroup_path;
@@ -528,9 +641,13 @@ again:
 			goto bad;
 	}
 	existed = 0;
-	for (i = 0; i < nr_subsystems; i++) {
-		if (!lxc_cgmanager_create(subsystems[i], tmp, &existed)) {
-			ERROR("Error creating cgroup %s:%s", subsystems[i], result);
+
+	if (cgm_supports_multiple_controllers)
+		slist = subsystems_inone;
+
+	for (i = 0; slist[i]; i++) {
+		if (!lxc_cgmanager_create(slist[i], tmp, &existed)) {
+			ERROR("Error creating cgroup %s:%s", slist[i], result);
 			cleanup_cgroups(tmp);
 			goto bad;
 		}
@@ -548,7 +665,6 @@ again:
 	return true;
 
 next:
-	cleanup_cgroups(tmp);
 	index++;
 	goto again;
 bad:
@@ -565,44 +681,50 @@ bad:
  * Internal helper, must be called with cgmanager dbus socket open
  */
 static bool lxc_cgmanager_enter(pid_t pid, const char *controller,
-		const char *cgroup_path)
+		const char *cgroup_path, bool abs)
 {
-	if (cgmanager_move_pid_sync(NULL, cgroup_manager, controller,
-			cgroup_path, pid) != 0) {
+	int ret;
+
+	if (abs)
+		ret = cgmanager_move_pid_abs_sync(NULL, cgroup_manager,
+			controller, cgroup_path, pid);
+	else
+		ret = cgmanager_move_pid_sync(NULL, cgroup_manager,
+			controller, cgroup_path, pid);
+	if (ret != 0) {
 		NihError *nerr;
 		nerr = nih_error_get();
-		ERROR("call to cgmanager_move_pid_sync failed: %s", nerr->message);
+		ERROR("call to cgmanager_move_pid_%ssync failed: %s",
+			abs ? "abs_" : "", nerr->message);
 		nih_free(nerr);
 		return false;
 	}
 	return true;
 }
 
-/* Internal helper, must be called with cgmanager dbus socket open */
-static bool do_cgm_enter(pid_t pid, const char *cgroup_path)
-{
-	int i;
-
-	for (i = 0; i < nr_subsystems; i++) {
-		if (!lxc_cgmanager_enter(pid, subsystems[i], cgroup_path))
-			return false;
-	}
-	return true;
-}
-
 static inline bool cgm_enter(void *hdata, pid_t pid)
 {
 	struct cgm_data *d = hdata;
+	char **slist = subsystems;
 	bool ret = false;
+	int i;
+
+	if (!d || !d->cgroup_path)
+		return false;
 
 	if (!cgm_dbus_connect()) {
 		ERROR("Error connecting to cgroup manager");
 		return false;
 	}
-	if (!d || !d->cgroup_path)
-		goto out;
-	if (do_cgm_enter(pid, d->cgroup_path))
-		ret = true;
+
+	if (cgm_all_controllers_same)
+		slist = subsystems_inone;
+
+	for (i = 0; slist[i]; i++) {
+		if (!lxc_cgmanager_enter(pid, slist[i], d->cgroup_path, false))
+			goto out;
+	}
+	ret = true;
 out:
 	cgm_dbus_disconnect();
 	return ret;
@@ -617,6 +739,42 @@ static const char *cgm_get_cgroup(void *hdata, const char *subsystem)
 	return d->cgroup_path;
 }
 
+#if HAVE_CGMANAGER_GET_PID_CGROUP_ABS_SYNC
+static inline bool abs_cgroup_supported(void) {
+	return api_version >= CGM_SUPPORTS_GET_ABS;
+}
+#else
+static inline bool abs_cgroup_supported(void) {
+	return false;
+}
+#define cgmanager_get_pid_cgroup_abs_sync(...) -1
+#endif
+
+static char *try_get_abs_cgroup(const char *name, const char *lxcpath,
+		const char *controller)
+{
+	char *cgroup = NULL;
+
+	if (abs_cgroup_supported()) {
+		/* get the container init pid and ask for its abs cgroup */
+		pid_t pid = lxc_cmd_get_init_pid(name, lxcpath);
+		if (pid < 0)
+			return NULL;
+		if (cgmanager_get_pid_cgroup_abs_sync(NULL, cgroup_manager,
+				controller, pid, &cgroup) != 0) {
+			cgroup = NULL;
+			NihError *nerr;
+			nerr = nih_error_get();
+			nih_free(nerr);
+		}
+		prune_init_scope(cgroup);
+		return cgroup;
+	}
+
+	/* use the command interface to look for the cgroup */
+	return lxc_cmd_get_cgroup_path(name, lxcpath, controller);
+}
+
 /*
  * nrtasks is called by the utmp helper by the container monitor.
  * cgmanager socket was closed after cgroup setup was complete, so we need
@@ -652,50 +810,167 @@ out:
 	return pids_len;
 }
 
-/* cgm_get is called to get container cgroup settings, not during startup */
-static int cgm_get(const char *filename, char *value, size_t len, const char *name, const char *lxcpath)
+#if HAVE_CGMANAGER_LIST_CONTROLLERS
+static bool lxc_list_controllers(char ***list)
 {
-	char *result, *controller, *key, *cgroup;
-	size_t newlen;
+	if (!cgm_dbus_connect()) {
+		ERROR("Error connecting to cgroup manager");
+		return false;
+	}
+	if (cgmanager_list_controllers_sync(NULL, cgroup_manager, list) != 0) {
+		NihError *nerr;
+		nerr = nih_error_get();
+		ERROR("call to cgmanager_list_controllers_sync failed: %s", nerr->message);
+		nih_free(nerr);
+		cgm_dbus_disconnect();
+		return false;
+	}
+
+	cgm_dbus_disconnect();
+	return true;
+}
+#else
+static bool lxc_list_controllers(char ***list)
+{
+	return false;
+}
+#endif
+
+static inline void free_abs_cgroup(char *cgroup)
+{
+	if (!cgroup)
+		return;
+	if (abs_cgroup_supported())
+		nih_free(cgroup);
+	else
+		free(cgroup);
+}
+
+static void do_cgm_get(const char *name, const char *lxcpath, const char *filename, int outp, bool sendvalue)
+{
+	char *controller, *key, *cgroup = NULL, *cglast;
+	int len = -1;
+	int ret;
+	nih_local char *result = NULL;
 
 	controller = alloca(strlen(filename)+1);
 	strcpy(controller, filename);
 	key = strchr(controller, '.');
-	if (!key)
-		return -1;
+	if (!key) {
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		exit(1);
+	}
 	*key = '\0';
 
-	/* use the command interface to look for the cgroup */
-	cgroup = lxc_cmd_get_cgroup_path(name, lxcpath, controller);
-	if (!cgroup)
-		return -1;
 	if (!cgm_dbus_connect()) {
 		ERROR("Error connecting to cgroup manager");
-		return -1;
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		exit(1);
 	}
-	if (cgmanager_get_value_sync(NULL, cgroup_manager, controller, cgroup, filename, &result) != 0) {
-		/*
-		 * must consume the nih error
-		 * However don't print out an error as the key may simply not exist
-		 * on the host
-		 */
+	cgroup = try_get_abs_cgroup(name, lxcpath, controller);
+	if (!cgroup) {
+		cgm_dbus_disconnect();
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		exit(1);
+	}
+	cglast = strrchr(cgroup, '/');
+	if (!cglast) {
+		cgm_dbus_disconnect();
+		free_abs_cgroup(cgroup);
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		exit(1);
+	}
+	*cglast = '\0';
+	if (!lxc_cgmanager_enter(getpid(), controller, cgroup, abs_cgroup_supported())) {
+		ERROR("Failed to enter container cgroup %s:%s", controller, cgroup);
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		cgm_dbus_disconnect();
+		free_abs_cgroup(cgroup);
+		exit(1);
+	}
+	if (cgmanager_get_value_sync(NULL, cgroup_manager, controller, cglast+1, filename, &result) != 0) {
 		NihError *nerr;
 		nerr = nih_error_get();
 		nih_free(nerr);
-		free(cgroup);
+		free_abs_cgroup(cgroup);
 		cgm_dbus_disconnect();
+		ret = write(outp, &len, sizeof(len));
+		if (ret != sizeof(len))
+			WARN("Failed to warn cgm_get of error; parent may hang");
+		exit(1);
+	}
+	free_abs_cgroup(cgroup);
+	cgm_dbus_disconnect();
+	len = strlen(result);
+	ret = write(outp, &len, sizeof(len));
+	if (ret != sizeof(len)) {
+		WARN("Failed to send length to parent");
+		exit(1);
+	}
+	if (!len || !sendvalue) {
+		exit(0);
+	}
+	ret = write(outp, result, len);
+	if (ret < 0)
+		exit(1);
+	exit(0);
+}
+
+/* cgm_get is called to get container cgroup settings, not during startup */
+static int cgm_get(const char *filename, char *value, size_t len, const char *name, const char *lxcpath)
+{
+	pid_t pid;
+	int p[2], ret, newlen, readlen;
+
+	if (pipe(p) < 0)
+		return -1;
+	if ((pid = fork()) < 0) {
+		close(p[0]);
+		close(p[1]);
 		return -1;
 	}
-	cgm_dbus_disconnect();
-	free(cgroup);
-	newlen = strlen(result);
+	if (!pid) // do_cgm_get exits
+		do_cgm_get(name, lxcpath, filename, p[1], len && value);
+	close(p[1]);
+	ret = read(p[0], &newlen, sizeof(newlen));
+	if (ret != sizeof(newlen)) {
+		close(p[0]);
+		ret = -1;
+		goto out;
+	}
 	if (!len || !value) {
-		// user queries the size
-		nih_free(result);
-		return newlen+1;
+		close(p[0]);
+		ret = newlen;
+		goto out;
+	}
+	memset(value, 0, len);
+	if (newlen < 0) { // child is reporting an error
+		close(p[0]);
+		ret = -1;
+		goto out;
+	}
+	if (newlen == 0) { // empty read
+		close(p[0]);
+		ret = 0;
+		goto out;
+	}
+	readlen = newlen > len ? len : newlen;
+	ret = read(p[0], value, readlen);
+	close(p[0]);
+	if (ret != readlen) {
+		ret = -1;
+		goto out;
 	}
-
-	strncpy(value, result, len);
 	if (newlen >= len) {
 		value[len-1] = '\0';
 		newlen = len-1;
@@ -704,56 +979,112 @@ static int cgm_get(const char *filename, char *value, size_t len, const char *na
 		value[newlen++] = '\n';
 		value[newlen] = '\0';
 	}
-	nih_free(result);
-	return newlen;
+	ret = newlen;
+out:
+	if (wait_for_pid(pid))
+		WARN("do_cgm_get exited with error");
+	return ret;
 }
 
-/* internal helper - call with cgmanager dbus connection open */
-static int cgm_do_set(const char *controller, const char *file,
-			 const char *cgroup, const char *value)
+static void do_cgm_set(const char *name, const char *lxcpath, const char *filename, const char *value, int outp)
 {
+	char *controller, *key, *cgroup = NULL;
+	int retval = 0;  // value we are sending to the parent over outp
 	int ret;
-	ret = cgmanager_set_value_sync(NULL, cgroup_manager, controller,
-				 cgroup, file, value);
-	if (ret != 0) {
+	char *cglast;
+
+	controller = alloca(strlen(filename)+1);
+	strcpy(controller, filename);
+	key = strchr(controller, '.');
+	if (!key) {
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		exit(1);
+	}
+	*key = '\0';
+
+	if (!cgm_dbus_connect()) {
+		ERROR("Error connecting to cgroup manager");
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		exit(1);
+	}
+	cgroup = try_get_abs_cgroup(name, lxcpath, controller);
+	if (!cgroup) {
+		cgm_dbus_disconnect();
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		exit(1);
+	}
+	cglast = strrchr(cgroup, '/');
+	if (!cglast) {
+		cgm_dbus_disconnect();
+		free_abs_cgroup(cgroup);
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		exit(1);
+	}
+	*cglast = '\0';
+	if (!lxc_cgmanager_enter(getpid(), controller, cgroup, abs_cgroup_supported())) {
+		ERROR("Failed to enter container cgroup %s:%s", controller, cgroup);
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		cgm_dbus_disconnect();
+		free_abs_cgroup(cgroup);
+		exit(1);
+	}
+	if (cgmanager_set_value_sync(NULL, cgroup_manager, controller, cglast+1, filename, value) != 0) {
 		NihError *nerr;
 		nerr = nih_error_get();
+		ERROR("Error setting cgroup value %s for %s:%s", filename, controller, cgroup);
 		ERROR("call to cgmanager_set_value_sync failed: %s", nerr->message);
 		nih_free(nerr);
-		ERROR("Error setting cgroup %s limit %s", file, cgroup);
+		free_abs_cgroup(cgroup);
+		cgm_dbus_disconnect();
+		ret = write(outp, &retval, sizeof(retval));
+		if (ret != sizeof(retval))
+			WARN("Failed to warn cgm_set of error; parent may hang");
+		exit(1);
 	}
-	return ret;
+	free_abs_cgroup(cgroup);
+	cgm_dbus_disconnect();
+	/* tell parent that we are done */
+	retval = 1;
+	ret = write(outp, &retval, sizeof(retval));
+	if (ret != sizeof(retval)) {
+		exit(1);
+	}
+	exit(0);
 }
 
 /* cgm_set is called to change cgroup settings, not during startup */
 static int cgm_set(const char *filename, const char *value, const char *name, const char *lxcpath)
 {
-	char *controller, *key, *cgroup;
-	int ret;
+	pid_t pid;
+	int p[2], ret, v;
 
-	controller = alloca(strlen(filename)+1);
-	strcpy(controller, filename);
-	key = strchr(controller, '.');
-	if (!key)
+	if (pipe(p) < 0)
 		return -1;
-	*key = '\0';
-
-	/* use the command interface to look for the cgroup */
-	cgroup = lxc_cmd_get_cgroup_path(name, lxcpath, controller);
-	if (!cgroup) {
-		ERROR("Failed to get cgroup for controller %s for %s:%s",
-			controller, lxcpath, name);
+	if ((pid = fork()) < 0) {
+		close(p[1]);
+		close(p[0]);
 		return -1;
 	}
-	if (!cgm_dbus_connect()) {
-		ERROR("Error connecting to cgroup manager");
-		free(cgroup);
-		return false;
-	}
-	ret = cgm_do_set(controller, filename, cgroup, value);
-	cgm_dbus_disconnect();
-	free(cgroup);
-	return ret;
+	if (!pid) // do_cgm_set exits
+		do_cgm_set(name, lxcpath, filename, value, p[1]);
+	close(p[1]);
+	ret = read(p[0], &v, sizeof(v));
+	close(p[0]);
+	if (wait_for_pid(pid))
+		WARN("do_cgm_set exited with error");
+	if (ret != sizeof(v) || !v)
+		return -1;
+	return 0;
 }
 
 static void free_subsystems(void)
@@ -767,50 +1098,189 @@ static void free_subsystems(void)
 	nr_subsystems = 0;
 }
 
+static void cull_user_controllers(void)
+{
+	int i, j;
+
+	for (i = 0;  i < nr_subsystems; i++) {
+		if (strncmp(subsystems[i], "name=", 5) != 0)
+			continue;
+		for (j = i;  j < nr_subsystems-1; j++)
+			subsystems[j] = subsystems[j+1];
+		nr_subsystems--;
+	}
+}
+
+static bool in_comma_list(const char *inword, const char *cgroup_use)
+{
+	char *e;
+	size_t inlen = strlen(inword), len;
+
+	do {
+		e = strchr(cgroup_use, ',');
+		len = e ? e - cgroup_use : strlen(cgroup_use);
+		if (len == inlen && strncmp(inword, cgroup_use, len) == 0)
+			return true;
+		cgroup_use = e + 1;
+	} while (e);
+
+	return false;
+}
+
+static bool in_subsystem_list(const char *c)
+{
+	int i;
+
+	for (i = 0; i < nr_subsystems; i++) {
+		if (strcmp(c, subsystems[i]) == 0)
+			return true;
+	}
+
+	return false;
+}
+
+/*
+ * If /etc/lxc/lxc.conf specifies lxc.cgroup.use = "freezer,memory",
+ * then clear out any other subsystems, and make sure that freezer
+ * and memory are both enabled
+ */
+static bool verify_and_prune(const char *cgroup_use)
+{
+	const char *p;
+	char *e;
+	int i, j;
+
+	for (p = cgroup_use; p && *p; p = e + 1) {
+		e = strchr(p, ',');
+		if (e)
+			*e = '\0';
+
+		if (!in_subsystem_list(p)) {
+			ERROR("Controller %s required by lxc.cgroup.use but not available\n", p);
+			return false;
+		}
+
+		if (e)
+			*e = ',';
+		if (!e)
+			break;
+	}
+
+	for (i = 0; i < nr_subsystems;) {
+		if (in_comma_list(subsystems[i], cgroup_use)) {
+			i++;
+			continue;
+		}
+		free(subsystems[i]);
+		for (j = i;  j < nr_subsystems-1; j++)
+			subsystems[j] = subsystems[j+1];
+		subsystems[nr_subsystems-1] = NULL;
+		nr_subsystems--;
+	}
+
+	return true;
+}
+
 static bool collect_subsytems(void)
 {
-	char *line = NULL, *tab1;
+	char *line = NULL;
+	nih_local char **cgm_subsys_list = NULL;
 	size_t sz = 0;
-	FILE *f;
+	FILE *f = NULL;
 
 	if (subsystems) // already initialized
 		return true;
 
-	f = fopen_cloexec("/proc/cgroups", "r");
-	if (!f) {
+	subsystems_inone = malloc(2 * sizeof(char *));
+	if (!subsystems_inone)
 		return false;
+	subsystems_inone[0] = "all";
+	subsystems_inone[1] = NULL;
+
+	if (lxc_list_controllers(&cgm_subsys_list)) {
+		while (cgm_subsys_list[nr_subsystems]) {
+			char **tmp = NIH_MUST( realloc(subsystems,
+						(nr_subsystems+2)*sizeof(char *)) );
+			tmp[nr_subsystems] = NIH_MUST(
+					strdup(cgm_subsys_list[nr_subsystems++]) );
+			subsystems = tmp;
+		}
+		if (nr_subsystems)
+			subsystems[nr_subsystems] = NULL;
+		goto collected;
+	}
+
+	INFO("cgmanager_list_controllers failed, falling back to /proc/self/cgroups");
+	f = fopen_cloexec("/proc/self/cgroup", "r");
+	if (!f) {
+		f = fopen_cloexec("/proc/1/cgroup", "r");
+		if (!f)
+			return false;
 	}
 	while (getline(&line, &sz, f) != -1) {
-		char **tmp;
-		if (line[0] == '#')
-			continue;
+		/* file format: hierarchy:subsystems:group,
+		 * with multiple subsystems being ,-separated */
+		char *slist, *end, *p, *saveptr = NULL, **tmp;
+
 		if (!line[0])
 			continue;
-		tab1 = strchr(line, '\t');
-		if (!tab1)
-			continue;
-		*tab1 = '\0';
-		tmp = realloc(subsystems, (nr_subsystems+1)*sizeof(char *));
-		if (!tmp)
-			goto out_free;
 
-		subsystems = tmp;
-		tmp[nr_subsystems] = strdup(line);
-		if (!tmp[nr_subsystems])
-			goto out_free;
-		nr_subsystems++;
+		slist = strchr(line, ':');
+		if (!slist)
+			continue;
+		slist++;
+		end = strchr(slist, ':');
+		if (!end)
+			continue;
+		*end = '\0';
+
+		for (p = strtok_r(slist, ",", &saveptr);
+				p;
+				p = strtok_r(NULL, ",", &saveptr)) {
+			tmp = realloc(subsystems, (nr_subsystems+2)*sizeof(char *));
+			if (!tmp)
+				goto out_free;
+
+			subsystems = tmp;
+			tmp[nr_subsystems] = strdup(p);
+			tmp[nr_subsystems+1] = NULL;
+			if (!tmp[nr_subsystems])
+				goto out_free;
+			nr_subsystems++;
+		}
 	}
 	fclose(f);
+	f = NULL;
 
+	free(line);
+	line = NULL;
+
+collected:
 	if (!nr_subsystems) {
 		ERROR("No cgroup subsystems found");
 		return false;
 	}
 
+	/* make sure that cgroup.use can be and is honored */
+	const char *cgroup_use = lxc_global_config_value("lxc.cgroup.use");
+	if (!cgroup_use && errno != 0)
+		goto out_good;
+	if (cgroup_use) {
+		if (!verify_and_prune(cgroup_use)) {
+			free_subsystems();
+			return false;
+		}
+		subsystems_inone[0] = NIH_MUST( strdup(cgroup_use) );
+		cgm_all_controllers_same = false;
+	}
+
+out_good:
 	return true;
 
 out_free:
-	fclose(f);
+	free(line);
+	if (f)
+		fclose(f);
 	free_subsystems();
 	return false;
 }
@@ -870,7 +1340,7 @@ static bool cgm_unfreeze(void *hdata)
 static bool cgm_setup_limits(void *hdata, struct lxc_list *cgroup_settings, bool do_devices)
 {
 	struct cgm_data *d = hdata;
-	struct lxc_list *iterator;
+	struct lxc_list *iterator, *sorted_cgroup_settings, *next;
 	struct lxc_cgroup *cg;
 	bool ret = false;
 
@@ -885,7 +1355,12 @@ static bool cgm_setup_limits(void *hdata, struct lxc_list *cgroup_settings, bool
 		return false;
 	}
 
-	lxc_list_for_each(iterator, cgroup_settings) {
+	sorted_cgroup_settings = sort_cgroup_settings(cgroup_settings);
+	if (!sorted_cgroup_settings) {
+		return false;
+	}
+
+	lxc_list_for_each(iterator, sorted_cgroup_settings) {
 		char controller[100], *p;
 		cg = iterator->elem;
 		if (do_devices != !strncmp("devices", cg->subsystem, 7))
@@ -896,10 +1371,14 @@ static bool cgm_setup_limits(void *hdata, struct lxc_list *cgroup_settings, bool
 		p = strchr(controller, '.');
 		if (p)
 			*p = '\0';
-		if (cgm_do_set(controller, cg->subsystem, d->cgroup_path
-				, cg->value) < 0) {
-			ERROR("Error setting %s to %s for %s",
-			      cg->subsystem, cg->value, d->name);
+		if (cgmanager_set_value_sync(NULL, cgroup_manager, controller,
+					 d->cgroup_path, cg->subsystem, cg->value) != 0) {
+			NihError *nerr;
+			nerr = nih_error_get();
+			ERROR("call to cgmanager_set_value_sync failed: %s", nerr->message);
+			nih_free(nerr);
+			ERROR("Error setting cgroup %s:%s limit type %s", controller,
+				d->cgroup_path, cg->subsystem);
 			goto out;
 		}
 
@@ -909,6 +1388,11 @@ static bool cgm_setup_limits(void *hdata, struct lxc_list *cgroup_settings, bool
 	ret = true;
 	INFO("cgroup limits have been setup");
 out:
+	lxc_list_for_each_safe(iterator, sorted_cgroup_settings, next) {
+		lxc_list_del(iterator);
+		free(iterator);
+	}
+	free(sorted_cgroup_settings);
 	cgm_dbus_disconnect();
 	return ret;
 }
@@ -916,7 +1400,6 @@ out:
 static bool cgm_chown(void *hdata, struct lxc_conf *conf)
 {
 	struct cgm_data *d = hdata;
-	int i;
 
 	if (!d || !d->cgroup_path)
 		return false;
@@ -924,11 +1407,8 @@ static bool cgm_chown(void *hdata, struct lxc_conf *conf)
 		ERROR("Error connecting to cgroup manager");
 		return false;
 	}
-	for (i = 0; i < nr_subsystems; i++) {
-		if (!chown_cgroup(subsystems[i], d->cgroup_path, conf))
-			WARN("Failed to chown %s:%s to container root",
-				subsystems[i], d->cgroup_path);
-	}
+	if (!chown_cgroup(d->cgroup_path, conf))
+		WARN("Failed to chown %s to container root", d->cgroup_path);
 	cgm_dbus_disconnect();
 	return true;
 }
@@ -943,36 +1423,35 @@ static bool cgm_chown(void *hdata, struct lxc_conf *conf)
  */
 static bool cgm_attach(const char *name, const char *lxcpath, pid_t pid)
 {
-	bool pass = false;
+	bool pass = true;
 	char *cgroup = NULL;
-	struct lxc_container *c;
-
-	c = lxc_container_new(name, lxcpath);
-	if (!c) {
-		ERROR("Could not load container %s:%s", lxcpath, name);
-		return false;
-	}
-	// cgm_create makes sure that we have the same cgroup name for all
-	// subsystems, so since this is a slow command over the cmd socket,
-	// just get the cgroup name for the first one.
-	cgroup = lxc_cmd_get_cgroup_path(name, lxcpath, subsystems[0]);
-	if (!cgroup) {
-		ERROR("Failed to get cgroup for controller %s", subsystems[0]);
-		goto out;
-	}
+	char **slist = subsystems;
+	int i;
 
 	if (!cgm_dbus_connect()) {
 		ERROR("Error connecting to cgroup manager");
-		goto out;
+		return false;
+	}
+
+	for (i = 0; slist[i]; i++) {
+		cgroup = try_get_abs_cgroup(name, lxcpath, slist[i]);
+		if (!cgroup) {
+			ERROR("Failed to get cgroup for controller %s", slist[i]);
+			cgm_dbus_disconnect();
+			return false;
+		}
+
+		if (!lxc_cgmanager_enter(pid, slist[i], cgroup, abs_cgroup_supported())) {
+			pass = false;
+			break;
+		}
+
 	}
-	pass = do_cgm_enter(pid, cgroup);
 	cgm_dbus_disconnect();
 	if (!pass)
 		ERROR("Failed to enter group %s", cgroup);
 
-out:
-	free(cgroup);
-	lxc_container_put(c);
+	free_abs_cgroup(cgroup);
 	return pass;
 }
 
@@ -990,7 +1469,7 @@ static bool cgm_bind_dir(const char *root, const char *dirname)
 	}
 
 	/* mount a tmpfs there so we can create subdirs */
-	if (mount("cgroup", cgpath, "tmpfs", 0, "size=10000,mode=755")) {
+	if (safe_mount("cgroup", cgpath, "tmpfs", 0, "size=10000,mode=755", root)) {
 		SYSERROR("Failed to mount tmpfs at %s", cgpath);
 		return false;
 	}
@@ -1001,7 +1480,7 @@ static bool cgm_bind_dir(const char *root, const char *dirname)
 		return false;
 	}
 
-	if (mount(dirname, cgpath, "none", MS_BIND, 0)) {
+	if (safe_mount(dirname, cgpath, "none", MS_BIND, 0, root)) {
 		SYSERROR("Failed to bind mount %s to %s", dirname, cgpath);
 		return false;
 	}
