@@ -173,3 +173,17 @@ void cgroup_disconnect(void)
 	if (ops && ops->disconnect)
 		ops->disconnect();
 }
+
+#define INIT_SCOPE "/init.scope"
+void prune_init_scope(char *cg)
+{
+	char *point = cg + strlen(cg) - strlen(INIT_SCOPE);
+	if (point < cg)
+		return;
+	if (strcmp(point, INIT_SCOPE) == 0) {
+		if (point == cg)
+			*(point+1) = '\0';
+		else
+			*point = '\0';
+	}
+}
