@@ -64,6 +64,8 @@ static int config_idmap(const char *, const char *, struct lxc_conf *);
 static int config_loglevel(const char *, const char *, struct lxc_conf *);
 static int config_logfile(const char *, const char *, struct lxc_conf *);
 static int config_mount(const char *, const char *, struct lxc_conf *);
+static int config_mount_auto(const char *, const char *, struct lxc_conf *);
+static int config_fstab(const char *, const char *, struct lxc_conf *);
 static int config_rootfs(const char *, const char *, struct lxc_conf *);
 static int config_rootfs_mount(const char *, const char *, struct lxc_conf *);
 static int config_rootfs_options(const char *, const char *, struct lxc_conf *);
@@ -88,6 +90,7 @@ static int config_network_ipv6_gateway(const char *, const char *, struct lxc_co
 static int config_cap_drop(const char *, const char *, struct lxc_conf *);
 static int config_cap_keep(const char *, const char *, struct lxc_conf *);
 static int config_console(const char *, const char *, struct lxc_conf *);
+static int config_console_logfile(const char *, const char *, struct lxc_conf *);
 static int config_seccomp(const char *, const char *, struct lxc_conf *);
 static int config_includefile(const char *, const char *, struct lxc_conf *);
 static int config_network_nic(const char *, const char *, struct lxc_conf *);
@@ -110,7 +113,9 @@ static struct lxc_config_t config[] = {
 	{ "lxc.id_map",               config_idmap                },
 	{ "lxc.loglevel",             config_loglevel             },
 	{ "lxc.logfile",              config_logfile              },
-	{ "lxc.mount",                config_mount                },
+	{ "lxc.mount.entry",          config_mount                },
+	{ "lxc.mount.auto",           config_mount_auto           },
+	{ "lxc.mount",                config_fstab                },
 	{ "lxc.rootfs.mount",         config_rootfs_mount         },
 	{ "lxc.rootfs.options",       config_rootfs_options       },
 	{ "lxc.rootfs",               config_rootfs               },
@@ -142,6 +147,7 @@ static struct lxc_config_t config[] = {
 	{ "lxc.network.",             config_network_nic          },
 	{ "lxc.cap.drop",             config_cap_drop             },
 	{ "lxc.cap.keep",             config_cap_keep             },
+	{ "lxc.console.logfile",      config_console_logfile      },
 	{ "lxc.console",              config_console              },
 	{ "lxc.seccomp",              config_seccomp              },
 	{ "lxc.include",              config_includefile          },
@@ -179,6 +185,63 @@ static const struct signame signames[] = {
 	{ SIGTSTP,   "TSTP" },
 	{ SIGTTIN,   "TTIN" },
 	{ SIGTTOU,   "TTOU" },
+#ifdef SIGTRAP
+	{ SIGTRAP,   "TRAP" },
+#endif
+#ifdef SIGIOT
+	{ SIGIOT,    "IOT" },
+#endif
+#ifdef SIGEMT
+	{ SIGEMT,    "EMT" },
+#endif
+#ifdef SIGBUS
+	{ SIGBUS,    "BUS" },
+#endif
+#ifdef SIGSTKFLT
+	{ SIGSTKFLT, "STKFLT" },
+#endif
+#ifdef SIGCLD
+	{ SIGCLD,    "CLD" },
+#endif
+#ifdef SIGURG
+	{ SIGURG,    "URG" },
+#endif
+#ifdef SIGXCPU
+	{ SIGXCPU,   "XCPU" },
+#endif
+#ifdef SIGXFSZ
+	{ SIGXFSZ,   "XFSZ" },
+#endif
+#ifdef SIGVTALRM
+	{ SIGVTALRM, "VTALRM" },
+#endif
+#ifdef SIGPROF
+	{ SIGPROF,   "PROF" },
+#endif
+#ifdef SIGWINCH
+	{ SIGWINCH,  "WINCH" },
+#endif
+#ifdef SIGIO
+	{ SIGIO,     "IO" },
+#endif
+#ifdef SIGPOLL
+	{ SIGPOLL,   "POLL" },
+#endif
+#ifdef SIGINFO
+	{ SIGINFO,   "INFO" },
+#endif
+#ifdef SIGLOST
+	{ SIGLOST,   "LOST" },
+#endif
+#ifdef SIGPWR
+	{ SIGPWR,    "PWR" },
+#endif
+#ifdef SIGUNUSED
+	{ SIGUNUSED, "UNUSED" },
+#endif
+#ifdef SIGSYS
+	{ SIGSYS,    "SYS" },
+#endif
 };
 
 static const size_t config_size = sizeof(config)/sizeof(struct lxc_config_t);
@@ -228,8 +291,7 @@ static int config_string_item(char **conf_item, const char *value)
 	char *new_value;
 
 	if (!value || strlen(value) == 0) {
-		if (*conf_item)
-			free(*conf_item);
+		free(*conf_item);
 		*conf_item = NULL;
 		return 0;
 	}
@@ -240,8 +302,7 @@ static int config_string_item(char **conf_item, const char *value)
 		return -1;
 	}
 
-	if (*conf_item)
-		free(*conf_item);
+	free(*conf_item);
 	*conf_item = new_value;
 	return 0;
 }
@@ -287,7 +348,7 @@ static int config_network_nic(const char *key, const char *value,
 	 */
 	if (*(key+12) < '0' || *(key+12) > '9')
 		goto out;
-	p = index(key+12, '.');
+	p = strchr(key+12, '.');
 	if (!p)
 		goto out;
 	strcpy(copy+12, p+1);
@@ -431,9 +492,9 @@ extern int lxc_list_nicconfigs(struct lxc_conf *c, const char *key,
 		strprint(retv, inlen, "hwaddr\n");
 		strprint(retv, inlen, "mtu\n");
 		strprint(retv, inlen, "ipv6\n");
-		strprint(retv, inlen, "ipv6_gateway\n");
+		strprint(retv, inlen, "ipv6.gateway\n");
 		strprint(retv, inlen, "ipv4\n");
-		strprint(retv, inlen, "ipv4_gateway\n");
+		strprint(retv, inlen, "ipv4.gateway\n");
 	}
 	switch(netdev->type) {
 	case LXC_NET_VETH:
@@ -595,6 +656,10 @@ static int config_network_veth_pair(const char *key, const char *value,
 	if (!netdev)
 		return -1;
 
+	if (netdev->type != LXC_NET_VETH) {
+		ERROR("Invalid veth pair for a non-veth netdev");
+		return -1;
+	}
 	return network_ifname(&netdev->priv.veth_attr.pair, value);
 }
 
@@ -607,6 +672,10 @@ static int config_network_macvlan_mode(const char *key, const char *value,
 	if (!netdev)
 		return -1;
 
+	if (netdev->type != LXC_NET_MACVLAN) {
+		ERROR("Invalid macvlan.mode for a non-macvlan netdev");
+		return -1;
+	}
 	return macvlan_mode(&netdev->priv.macvlan_attr.mode, value);
 }
 
@@ -647,6 +716,10 @@ static int config_network_vlan_id(const char *key, const char *value,
 	if (!netdev)
 		return -1;
 
+	if (netdev->type != LXC_NET_VLAN) {
+		ERROR("Invalid vlan.id for a non-macvlan netdev");
+		return -1;
+	}
 	if (get_u16(&netdev->priv.vlan_attr.vid, value, 0))
 		return -1;
 
@@ -753,29 +826,27 @@ static int config_network_ipv4_gateway(const char *key, const char *value,
 			               struct lxc_conf *lxc_conf)
 {
 	struct lxc_netdev *netdev;
-	struct in_addr *gw;
 
 	netdev = network_netdev(key, value, &lxc_conf->network);
 	if (!netdev)
 		return -1;
 
-	gw = malloc(sizeof(*gw));
-	if (!gw) {
-		SYSERROR("failed to allocate ipv4 gateway address");
-		return -1;
-	}
+	free(netdev->ipv4_gateway);
 
-	if (!value) {
-		ERROR("no ipv4 gateway address specified");
-		free(gw);
-		return -1;
-	}
-
-	if (!strcmp(value, "auto")) {
-		free(gw);
+	if (!value || strlen(value) == 0) {
+		netdev->ipv4_gateway = NULL;
+	} else if (!strcmp(value, "auto")) {
 		netdev->ipv4_gateway = NULL;
 		netdev->ipv4_gateway_auto = true;
 	} else {
+		struct in_addr *gw;
+
+		gw = malloc(sizeof(*gw));
+		if (!gw) {
+			SYSERROR("failed to allocate ipv4 gateway address");
+			return -1;
+		}
+
 		if (!inet_pton(AF_INET, value, gw)) {
 			SYSERROR("invalid ipv4 gateway address: %s", value);
 			free(gw);
@@ -858,12 +929,11 @@ static int config_network_ipv6_gateway(const char *key, const char *value,
 	if (!netdev)
 		return -1;
 
-	if (!value) {
-		ERROR("no ipv6 gateway address specified");
-		return -1;
-	}
+	free(netdev->ipv6_gateway);
 
-	if (!strcmp(value, "auto")) {
+	if (!value || strlen(value) == 0) {
+		netdev->ipv4_gateway = NULL;
+	} else if (!strcmp(value, "auto")) {
 		netdev->ipv6_gateway = NULL;
 		netdev->ipv6_gateway_auto = true;
 	} else {
@@ -1025,10 +1095,10 @@ static int config_group(const char *key, const char *value,
 	/* in case several groups are specified in a single line
 	 * split these groups in a single element for the list */
 	for (groupptr = groups;;groupptr = NULL) {
-                token = strtok_r(groupptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(groupptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		grouplist = malloc(sizeof(*grouplist));
@@ -1045,7 +1115,7 @@ static int config_group(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->groups, grouplist);
-        }
+	}
 
 	free(groups);
 
@@ -1165,10 +1235,10 @@ static const char *sig_name(int signum) {
 	int n;
 
 	for (n = 0; n < sizeof(signames) / sizeof((signames)[0]); n++) {
-		if (n == signames[n].num)
+		if (signum == signames[n].num)
 			return signames[n].name;
 	}
-	return "";
+	return NULL;
 }
 
 static int sig_parse(const char *signame) {
@@ -1258,15 +1328,12 @@ static int config_cgroup(const char *key, const char *value,
 	return 0;
 
 out:
-	if (cglist)
-		free(cglist);
+	free(cglist);
 
 	if (cgelem) {
-		if (cgelem->subsystem)
-			free(cgelem->subsystem);
+		free(cgelem->subsystem);
 
-		if (cgelem->value)
-			free(cgelem->value);
+		free(cgelem->value);
 
 		free(cgelem);
 	}
@@ -1326,8 +1393,7 @@ static int config_idmap(const char *key, const char *value, struct lxc_conf *lxc
 	return 0;
 
 out:
-	if (idmaplist)
-		free(idmaplist);
+	free(idmaplist);
 
 	if (idmap) {
 		free(idmap);
@@ -1339,6 +1405,8 @@ out:
 static int config_fstab(const char *key, const char *value,
 			struct lxc_conf *lxc_conf)
 {
+	if (!value || strlen(value) == 0)
+		return -1;
 	return config_path_item(&lxc_conf->fstab, value);
 }
 
@@ -1347,21 +1415,21 @@ static int config_mount_auto(const char *key, const char *value,
 {
 	char *autos, *autoptr, *sptr, *token;
 	static struct { const char *token; int mask; int flag; } allowed_auto_mounts[] = {
-		{ "proc",               LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_MIXED        },
-		{ "proc:mixed",         LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_MIXED        },
-		{ "proc:rw",            LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_RW           },
-		{ "sys",                LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RO            },
-		{ "sys:ro",             LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RO            },
-		{ "sys:rw",             LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RW            },
-		{ "cgroup",             LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_MIXED      },
-		{ "cgroup:mixed",       LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_MIXED      },
-		{ "cgroup:ro",          LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_RO         },
-		{ "cgroup:rw",          LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_RW         },
-		{ "cgroup-full",        LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_MIXED },
-		{ "cgroup-full:mixed",  LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_MIXED },
-		{ "cgroup-full:ro",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RO    },
-		{ "cgroup-full:rw",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RW    },
-		/* NB: For adding anything that ist just a single on/off, but has
+		{ "proc",               LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_MIXED         },
+		{ "proc:mixed",         LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_MIXED         },
+		{ "proc:rw",            LXC_AUTO_PROC_MASK,      LXC_AUTO_PROC_RW            },
+		{ "sys",                LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RO             },
+		{ "sys:ro",             LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RO             },
+		{ "sys:rw",             LXC_AUTO_SYS_MASK,       LXC_AUTO_SYS_RW             },
+		{ "cgroup",             LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_NOSPEC      },
+		{ "cgroup:mixed",       LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_MIXED       },
+		{ "cgroup:ro",          LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_RO          },
+		{ "cgroup:rw",          LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_RW          },
+		{ "cgroup-full",        LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_NOSPEC },
+		{ "cgroup-full:mixed",  LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_MIXED  },
+		{ "cgroup-full:ro",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RO     },
+		{ "cgroup-full:rw",     LXC_AUTO_CGROUP_MASK,    LXC_AUTO_CGROUP_FULL_RW     },
+		/* NB: For adding anything that is just a single on/off, but has
 		 *     no options: keep mask and flag identical and just define the
 		 *     enum value as an unused bit so far
 		 */
@@ -1370,8 +1438,10 @@ static int config_mount_auto(const char *key, const char *value,
 	int i;
 	int ret = -1;
 
-	if (!strlen(value))
-		return -1;
+	if (!value || strlen(value) == 0) {
+		lxc_conf->auto_mounts = 0;
+		return 0;
+	}
 
 	autos = strdup(value);
 	if (!autos) {
@@ -1380,10 +1450,10 @@ static int config_mount_auto(const char *key, const char *value,
 	}
 
 	for (autoptr = autos; ; autoptr = NULL) {
-                token = strtok_r(autoptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(autoptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		for (i = 0; allowed_auto_mounts[i].token; i++) {
@@ -1398,7 +1468,7 @@ static int config_mount_auto(const char *key, const char *value,
 
 		lxc_conf->auto_mounts &= ~allowed_auto_mounts[i].mask;
 		lxc_conf->auto_mounts |= allowed_auto_mounts[i].flag;
-        }
+	}
 
 	free(autos);
 
@@ -1408,36 +1478,12 @@ static int config_mount_auto(const char *key, const char *value,
 static int config_mount(const char *key, const char *value,
 			struct lxc_conf *lxc_conf)
 {
-	char *fstab_token = "lxc.mount";
-	char *token = "lxc.mount.entry";
-	char *auto_token = "lxc.mount.auto";
-	char *subkey;
 	char *mntelem;
 	struct lxc_list *mntlist;
 
 	if (!value || strlen(value) == 0)
 		return lxc_clear_mount_entries(lxc_conf);
 
-	subkey = strstr(key, token);
-
-	if (!subkey) {
-		subkey = strstr(key, auto_token);
-
-		if (!subkey) {
-			subkey = strstr(key, fstab_token);
-
-			if (!subkey)
-				return -1;
-
-			return config_fstab(key, value, lxc_conf);
-		}
-
-		return config_mount_auto(key, value, lxc_conf);
-	}
-
-	if (!strlen(subkey))
-		return -1;
-
 	mntlist = malloc(sizeof(*mntlist));
 	if (!mntlist)
 		return -1;
@@ -1473,10 +1519,10 @@ static int config_cap_keep(const char *key, const char *value,
 	/* in case several capability keep is specified in a single line
 	 * split these caps in a single element for the list */
 	for (keepptr = keepcaps;;keepptr = NULL) {
-                token = strtok_r(keepptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(keepptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		keeplist = malloc(sizeof(*keeplist));
@@ -1493,7 +1539,7 @@ static int config_cap_keep(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->keepcaps, keeplist);
-        }
+	}
 
 	free(keepcaps);
 
@@ -1519,10 +1565,10 @@ static int config_cap_drop(const char *key, const char *value,
 	/* in case several capability drop is specified in a single line
 	 * split these caps in a single element for the list */
 	for (dropptr = dropcaps;;dropptr = NULL) {
-                token = strtok_r(dropptr, " \t", &sptr);
-                if (!token) {
+		token = strtok_r(dropptr, " \t", &sptr);
+		if (!token) {
 			ret = 0;
-                        break;
+			break;
 		}
 
 		droplist = malloc(sizeof(*droplist));
@@ -1539,7 +1585,7 @@ static int config_cap_drop(const char *key, const char *value,
 		}
 
 		lxc_list_add_tail(&lxc_conf->caps, droplist);
-        }
+	}
 
 	free(dropcaps);
 
@@ -1552,6 +1598,12 @@ static int config_console(const char *key, const char *value,
 	return config_path_item(&lxc_conf->console.path, value);
 }
 
+static int config_console_logfile(const char *key, const char *value,
+			  struct lxc_conf *lxc_conf)
+{
+	return config_path_item(&lxc_conf->console.log_path, value);
+}
+
 static int config_includefile(const char *key, const char *value,
 			  struct lxc_conf *lxc_conf)
 {
@@ -1601,8 +1653,7 @@ static int config_utsname(const char *key, const char *value,
 	}
 
 	strcpy(utsname->nodename, value);
-	if (lxc_conf->utsname)
-		free(lxc_conf->utsname);
+	free(lxc_conf->utsname);
 	lxc_conf->utsname = utsname;
 
 	return 0;
@@ -1723,9 +1774,15 @@ signed long lxc_config_parse_arch(const char *arch)
 	struct per_name {
 		char *name;
 		unsigned long per;
-	} pername[4] = {
+	} pername[] = {
 		{ "x86", PER_LINUX32 },
+		{ "linux32", PER_LINUX32 },
+		{ "i386", PER_LINUX32 },
+		{ "i486", PER_LINUX32 },
+		{ "i586", PER_LINUX32 },
 		{ "i686", PER_LINUX32 },
+		{ "athlon", PER_LINUX32 },
+		{ "linux64", PER_LINUX },
 		{ "x86_64", PER_LINUX },
 		{ "amd64", PER_LINUX },
 	};
@@ -1757,7 +1814,7 @@ int lxc_fill_elevated_privileges(char *flaglist, int *flags)
 		/* for the sake of backward compatibility, drop all privileges
 		   if none is specified */
 		for (i = 0; all_privs[i].token; i++) {
-	                *flags |= all_privs[i].flag;
+			*flags |= all_privs[i].flag;
 		}
 		return 0;
 	}
@@ -1854,8 +1911,8 @@ static int lxc_get_item_hooks(struct lxc_conf *c, char *retv, int inlen,
 	int i;
 
 	/* "lxc.hook.mount" */
-	subkey = index(key, '.');
-	if (subkey) subkey = index(subkey+1, '.');
+	subkey = strchr(key, '.');
+	if (subkey) subkey = strchr(subkey+1, '.');
 	if (!subkey)
 		return -1;
 	subkey++;
@@ -1948,6 +2005,7 @@ static int lxc_get_mount_entries(struct lxc_conf *c, char *retv, int inlen)
 static int lxc_get_auto_mounts(struct lxc_conf *c, char *retv, int inlen)
 {
 	int len, fulllen = 0;
+	const char *sep = "";
 
 	if (!retv)
 		inlen = 0;
@@ -1958,31 +2016,34 @@ static int lxc_get_auto_mounts(struct lxc_conf *c, char *retv, int inlen)
 		return 0;
 
 	switch (c->auto_mounts & LXC_AUTO_PROC_MASK) {
-		case LXC_AUTO_PROC_MIXED:        strprint(retv, inlen, "proc:mixed\n"); break;
-		case LXC_AUTO_PROC_RW:           strprint(retv, inlen, "proc:rw");      break;
+		case LXC_AUTO_PROC_MIXED:         strprint(retv, inlen, "%sproc:mixed", sep);        sep = " "; break;
+		case LXC_AUTO_PROC_RW:            strprint(retv, inlen, "%sproc:rw", sep);           sep = " "; break;
 		default: break;
 	}
 	switch (c->auto_mounts & LXC_AUTO_SYS_MASK) {
-		case LXC_AUTO_SYS_RO:            strprint(retv, inlen, "sys:ro");            break;
-		case LXC_AUTO_SYS_RW:            strprint(retv, inlen, "sys:rw");            break;
+		case LXC_AUTO_SYS_RO:             strprint(retv, inlen, "%ssys:ro", sep);            sep = " "; break;
+		case LXC_AUTO_SYS_RW:             strprint(retv, inlen, "%ssys:rw", sep);            sep = " "; break;
 		default: break;
 	}
 	switch (c->auto_mounts & LXC_AUTO_CGROUP_MASK) {
-		case LXC_AUTO_CGROUP_MIXED:      strprint(retv, inlen, "cgroup:mixed");      break;
-		case LXC_AUTO_CGROUP_RO:         strprint(retv, inlen, "cgroup:ro");         break;
-		case LXC_AUTO_CGROUP_RW:         strprint(retv, inlen, "cgroup:rw");         break;
-		case LXC_AUTO_CGROUP_FULL_MIXED: strprint(retv, inlen, "cgroup-full:mixed"); break;
-		case LXC_AUTO_CGROUP_FULL_RO:    strprint(retv, inlen, "cgroup-full:ro");    break;
-		case LXC_AUTO_CGROUP_FULL_RW:    strprint(retv, inlen, "cgroup-full:rw");    break;
+		case LXC_AUTO_CGROUP_NOSPEC:      strprint(retv, inlen, "%scgroup", sep);            sep = " "; break;
+		case LXC_AUTO_CGROUP_MIXED:       strprint(retv, inlen, "%scgroup:mixed", sep);      sep = " "; break;
+		case LXC_AUTO_CGROUP_RO:          strprint(retv, inlen, "%scgroup:ro", sep);         sep = " "; break;
+		case LXC_AUTO_CGROUP_RW:          strprint(retv, inlen, "%scgroup:rw", sep);         sep = " "; break;
+		case LXC_AUTO_CGROUP_FULL_NOSPEC: strprint(retv, inlen, "%scgroup-full", sep);       sep = " "; break;
+		case LXC_AUTO_CGROUP_FULL_MIXED:  strprint(retv, inlen, "%scgroup-full:mixed", sep); sep = " "; break;
+		case LXC_AUTO_CGROUP_FULL_RO:     strprint(retv, inlen, "%scgroup-full:ro", sep);    sep = " "; break;
+		case LXC_AUTO_CGROUP_FULL_RW:     strprint(retv, inlen, "%scgroup-full:rw", sep);    sep = " "; break;
 		default: break;
 	}
+
 	return fulllen;
 }
 
 /*
  * lxc.network.0.XXX, where XXX can be: name, type, link, flags, type,
  * macvlan.mode, veth.pair, vlan, ipv4, ipv6, script.up, hwaddr, mtu,
- * ipv4_gateway, ipv6_gateway.  ipvX_gateway can return 'auto' instead
+ * ipv4.gateway, ipv6.gateway.  ipvX.gateway can return 'auto' instead
  * of an address.  ipv4 and ipv6 return lists (newline-separated).
  * things like veth.pair return '' if invalid (i.e. if called for vlan
  * type).
@@ -1999,7 +2060,7 @@ static int lxc_get_item_nic(struct lxc_conf *c, char *retv, int inlen,
 	else
 		memset(retv, 0, inlen);
 
-	p1 = index(key, '.');
+	p1 = strchr(key, '.');
 	if (!p1 || *(p1+1) == '\0') return -1;
 	p1++;
 
@@ -2051,7 +2112,7 @@ static int lxc_get_item_nic(struct lxc_conf *c, char *retv, int inlen,
 		if (netdev->type == LXC_NET_VLAN) {
 			strprint(retv, inlen, "%d", netdev->priv.vlan_attr.vid);
 		}
-	} else if (strcmp(p1, "ipv4_gateway") == 0) {
+	} else if (strcmp(p1, "ipv4.gateway") == 0) {
 		if (netdev->ipv4_gateway_auto) {
 			strprint(retv, inlen, "auto");
 		} else if (netdev->ipv4_gateway) {
@@ -2065,23 +2126,23 @@ static int lxc_get_item_nic(struct lxc_conf *c, char *retv, int inlen,
 			struct lxc_inetdev *i = it2->elem;
 			char buf[INET_ADDRSTRLEN];
 			inet_ntop(AF_INET, &i->addr, buf, sizeof(buf));
-			strprint(retv, inlen, "%s\n", buf);
+			strprint(retv, inlen, "%s/%d\n", buf, i->prefix);
 		}
-	} else if (strcmp(p1, "ipv6_gateway") == 0) {
+	} else if (strcmp(p1, "ipv6.gateway") == 0) {
 		if (netdev->ipv6_gateway_auto) {
 			strprint(retv, inlen, "auto");
 		} else if (netdev->ipv6_gateway) {
-			char buf[INET_ADDRSTRLEN];
-			inet_ntop(AF_INET, netdev->ipv6_gateway, buf, sizeof(buf));
+			char buf[INET6_ADDRSTRLEN];
+			inet_ntop(AF_INET6, netdev->ipv6_gateway, buf, sizeof(buf));
 			strprint(retv, inlen, "%s", buf);
 		}
 	} else if (strcmp(p1, "ipv6") == 0) {
 		struct lxc_list *it2;
 		lxc_list_for_each(it2, &netdev->ipv6) {
-			struct lxc_inetdev *i = it2->elem;
-			char buf[INET_ADDRSTRLEN];
+			struct lxc_inet6dev *i = it2->elem;
+			char buf[INET6_ADDRSTRLEN];
 			inet_ntop(AF_INET6, &i->addr, buf, sizeof(buf));
-			strprint(retv, inlen, "%s\n", buf);
+			strprint(retv, inlen, "%s/%d\n", buf, i->prefix);
 		}
 	}
 	return fulllen;
@@ -2138,6 +2199,8 @@ int lxc_get_config_item(struct lxc_conf *c, const char *key, char *retv,
 		return lxc_get_cgroup_entry(c, retv, inlen, key + 11);
 	else if (strcmp(key, "lxc.utsname") == 0)
 		v = c->utsname ? c->utsname->nodename : NULL;
+	else if (strcmp(key, "lxc.console.logfile") == 0)
+		v = c->console.log_path;
 	else if (strcmp(key, "lxc.console") == 0)
 		v = c->console.path;
 	else if (strcmp(key, "lxc.rootfs.mount") == 0)
@@ -2201,6 +2264,9 @@ int lxc_clear_config_item(struct lxc_conf *c, const char *key)
 		lxc_seccomp_free(c);
 		return 0;
 	}
+	else if (strncmp(key, "lxc.id_map", 10) == 0) {
+		return lxc_clear_idmaps(c);
+	}
 
 	return -1;
 }
@@ -2212,6 +2278,7 @@ void write_config(FILE *fout, struct lxc_conf *c)
 {
 	struct lxc_list *it;
 	int i;
+	const char *signame;
 
 	if (c->fstab)
 		fprintf(fout, "lxc.mount = %s\n", c->fstab);
@@ -2221,22 +2288,24 @@ void write_config(FILE *fout, struct lxc_conf *c)
 	if (c->auto_mounts & LXC_AUTO_ALL_MASK) {
 		fprintf(fout, "lxc.mount.auto =");
 		switch (c->auto_mounts & LXC_AUTO_PROC_MASK) {
-			case LXC_AUTO_PROC_MIXED:        fprintf(fout, " proc:mixed");        break;
-			case LXC_AUTO_PROC_RW:           fprintf(fout, " proc:rw");           break;
+			case LXC_AUTO_PROC_MIXED:         fprintf(fout, " proc:mixed");        break;
+			case LXC_AUTO_PROC_RW:            fprintf(fout, " proc:rw");           break;
 			default: break;
 		}
 		switch (c->auto_mounts & LXC_AUTO_SYS_MASK) {
-			case LXC_AUTO_SYS_RO:            fprintf(fout, " sys:ro");            break;
-			case LXC_AUTO_SYS_RW:            fprintf(fout, " sys:rw");            break;
+			case LXC_AUTO_SYS_RO:             fprintf(fout, " sys:ro");            break;
+			case LXC_AUTO_SYS_RW:             fprintf(fout, " sys:rw");            break;
 			default: break;
 		}
 		switch (c->auto_mounts & LXC_AUTO_CGROUP_MASK) {
-			case LXC_AUTO_CGROUP_MIXED:      fprintf(fout, " cgroup:mixed");      break;
-			case LXC_AUTO_CGROUP_RO:         fprintf(fout, " cgroup:ro");         break;
-			case LXC_AUTO_CGROUP_RW:         fprintf(fout, " cgroup:rw");         break;
-			case LXC_AUTO_CGROUP_FULL_MIXED: fprintf(fout, " cgroup-full:mixed"); break;
-			case LXC_AUTO_CGROUP_FULL_RO:    fprintf(fout, " cgroup-full:ro");    break;
-			case LXC_AUTO_CGROUP_FULL_RW:    fprintf(fout, " cgroup-full:rw");    break;
+			case LXC_AUTO_CGROUP_NOSPEC:      fprintf(fout, " cgroup");            break;
+			case LXC_AUTO_CGROUP_MIXED:       fprintf(fout, " cgroup:mixed");      break;
+			case LXC_AUTO_CGROUP_RO:          fprintf(fout, " cgroup:ro");         break;
+			case LXC_AUTO_CGROUP_RW:          fprintf(fout, " cgroup:rw");         break;
+			case LXC_AUTO_CGROUP_FULL_NOSPEC: fprintf(fout, " cgroup-full");       break;
+			case LXC_AUTO_CGROUP_FULL_MIXED:  fprintf(fout, " cgroup-full:mixed"); break;
+			case LXC_AUTO_CGROUP_FULL_RO:     fprintf(fout, " cgroup-full:ro");    break;
+			case LXC_AUTO_CGROUP_FULL_RW:     fprintf(fout, " cgroup-full:rw");    break;
 			default: break;
 		}
 		fprintf(fout, "\n");
@@ -2247,10 +2316,22 @@ void write_config(FILE *fout, struct lxc_conf *c)
 		fprintf(fout, "lxc.pts = %d\n", c->pts);
 	if (c->ttydir)
 		fprintf(fout, "lxc.devttydir = %s\n", c->ttydir);
-	if (c->haltsignal)
-		fprintf(fout, "lxc.haltsignal = SIG%s\n", sig_name(c->haltsignal));
-	if (c->stopsignal)
-		fprintf(fout, "lxc.stopsignal = SIG%s\n", sig_name(c->stopsignal));
+	if (c->haltsignal) {
+		signame = sig_name(c->haltsignal);
+		if (signame == NULL) {
+			fprintf(fout, "lxc.haltsignal = %d\n", c->haltsignal);
+		} else {
+			fprintf(fout, "lxc.haltsignal = SIG%s\n", signame);
+		}
+	}
+	if (c->stopsignal) {
+		signame = sig_name(c->stopsignal);
+		if (signame == NULL) {
+			fprintf(fout, "lxc.stopsignal = %d\n", c->stopsignal);
+		} else {
+			fprintf(fout, "lxc.stopsignal = SIG%s\n", signame);
+		}
+	}
 	#if HAVE_SYS_PERSONALITY_H
 	switch(c->personality) {
 	case PER_LINUX32: fprintf(fout, "lxc.arch = x86\n"); break;
@@ -2373,6 +2454,8 @@ void write_config(FILE *fout, struct lxc_conf *c)
 	}
 	if (c->console.path)
 		fprintf(fout, "lxc.console = %s\n", c->console.path);
+	if (c->console.log_path)
+		fprintf(fout, "lxc.console.logfile = %s\n", c->console.log_path);
 	if (c->rootfs.path)
 		fprintf(fout, "lxc.rootfs = %s\n", c->rootfs.path);
 	if (c->rootfs.mount && strcmp(c->rootfs.mount, LXCROOTFSMOUNT) != 0)
