@@ -20,8 +20,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
-#ifndef __lxc_h
-#define __lxc_h
+#ifndef __LXC_LXC_H
+#define __LXC_LXC_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -47,7 +47,7 @@ struct lxc_arguments;
  * @name     : the name of the container
  * @argv     : an array of char * corresponding to the commande line
  * @conf     : configuration
- * Returns 0 on sucess, < 0 otherwise
+ * Returns 0 on success, < 0 otherwise
  */
 extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf,
 		     const char *lxcpath);
@@ -58,49 +58,11 @@ extern int lxc_start(const char *name, char *const argv[], struct lxc_conf *conf
  * @argv     : an array of char * corresponding to the commande line
  * @quiet    : if != 0 then lxc-init won't produce any output
  * @conf     : configuration
- * Returns 0 on sucess, < 0 otherwise
+ * Returns 0 on success, < 0 otherwise
  */
 extern int lxc_execute(const char *name, char *const argv[], int quiet,
 		       struct lxc_conf *conf, const char *lxcpath);
 
-/*
- * Open the monitoring mechanism for a specific container
- * The function will return an fd corresponding to the events
- * Returns a file descriptor on success, < 0 otherwise
- */
-extern int lxc_monitor_open(const char *lxcpath);
-
-/*
- * Blocking read for the next container state change
- * @fd  : the file descriptor provided by lxc_monitor_open
- * @msg : the variable which will be filled with the state
- * Returns 0 if the monitored container has exited, > 0 if
- * data was read, < 0 otherwise
- */
-extern int lxc_monitor_read(int fd, struct lxc_msg *msg);
-
-/*
- * Blocking read for the next container state change with timeout
- * @fd      : the file descriptor provided by lxc_monitor_open
- * @msg     : the variable which will be filled with the state
- * @timeout : the timeout in seconds to wait for a state change
- * Returns 0 if the monitored container has exited, > 0 if
- * data was read, < 0 otherwise
- */
-extern int lxc_monitor_read_timeout(int fd, struct lxc_msg *msg, int timeout);
-
-/*
- * Blocking read from multiple monitors for the next container state
- * change with timeout
- * @rfds    : an fd_set of file descriptors provided by lxc_monitor_open
- * @nfds    : the maximum fd number in rfds + 1
- * @msg     : the variable which will be filled with the state
- * @timeout : the timeout in seconds to wait for a state change
- * Returns 0 if the monitored container has exited, > 0 if
- * data was read, < 0 otherwise
- */
-extern int lxc_monitor_read_fdset(fd_set *rfds, int nfds, struct lxc_msg *msg, int timeout);
-
 /*
  * Close the fd associated with the monitoring
  * @fd : the file descriptor provided by lxc_monitor_open
@@ -118,7 +80,7 @@ extern int lxc_freeze(const char *name, const char *lxcpath);
 /*
  * Unfreeze all previously frozen tasks.
  * @name : the name of the container
- * Return 0 on sucess, < 0 otherwise
+ * Return 0 on success, < 0 otherwise
  */
 extern int lxc_unfreeze(const char *name, const char *lxcpath);
 
