@@ -133,8 +133,7 @@ static int get_alloted(char *me, char *intype, char *link)
 		return n;
 	}
 	fclose(fin);
-	if (line)
-		free(line);
+	free(line);
 	return -1;
 }
 
@@ -197,7 +196,7 @@ static bool nic_exists(char *nic)
 	return true;
 }
 
-static int instanciate_veth(char *n1, char **n2)
+static int instantiate_veth(char *n1, char **n2)
 {
 	int err;
 
@@ -246,7 +245,7 @@ static bool create_nic(char *nic, char *br, int pid, char **cnic)
 	}
 
 	/* create the nics */
-	if (instanciate_veth(veth1buf, &veth2buf) < 0) {
+	if (instantiate_veth(veth1buf, &veth2buf) < 0) {
 		fprintf(stderr, "Error creating veth tunnel\n");
 		return false;
 	}
@@ -268,7 +267,7 @@ static bool create_nic(char *nic, char *br, int pid, char **cnic)
 	}
 
 	/* pass veth2 to target netns */
-	ret = lxc_netdev_move_by_name(veth2buf, pid);
+	ret = lxc_netdev_move_by_name(veth2buf, pid, NULL);
 	if (ret < 0) {
 		fprintf(stderr, "Error moving %s to netns %d\n", veth2buf, pid);
 		goto out_del;
@@ -286,13 +285,16 @@ out_del:
  * *dest will container the name (vethXXXXXX) which is attached
  * on the host to the lxc bridge
  */
-static void get_new_nicname(char **dest, char *br, int pid, char **cnic)
+static bool get_new_nicname(char **dest, char *br, int pid, char **cnic)
 {
 	char template[IFNAMSIZ];
 	snprintf(template, sizeof(template), "vethXXXXXX");
 	*dest = lxc_mkifname(template);
 
-	create_nic(*dest, br, pid, cnic);
+	if (!create_nic(*dest, br, pid, cnic)) {
+		return false;
+	}
+	return true;
 }
 
 static bool get_nic_from_line(char *p, char **nic)
@@ -419,7 +421,8 @@ static bool get_nic_if_avail(int fd, char *me, int pid, char *intype, char *br,
 			return false;
 	}
 
-	get_new_nicname(nicname, br, pid, cnic);
+	if (!get_new_nicname(nicname, br, pid, cnic))
+		return false;
 	/* me  ' ' intype ' ' br ' ' *nicname + '\n' + '\0' */
 	slen = strlen(me) + strlen(intype) + strlen(br) + strlen(*nicname) + 5;
 	newline = alloca(slen);
@@ -466,10 +469,13 @@ again:
 	goto again;
 }
 
-static int rename_in_ns(int pid, char *oldname, char *newname)
+#define VETH_DEF_NAME "eth%d"
+
+static int rename_in_ns(int pid, char *oldname, char **newnamep)
 {
 	char nspath[MAXPATHLEN];
-	int fd = -1, ofd = -1, ret;
+	int fd = -1, ofd = -1, ret, ifindex = -1;
+	bool grab_newname = false;
 
 	ret = snprintf(nspath, MAXPATHLEN, "/proc/%d/ns/net", getpid());
 	if (ret < 0 || ret >= MAXPATHLEN)
@@ -491,10 +497,28 @@ static int rename_in_ns(int pid, char *oldname, char *newname)
 		goto out_err;
 	}
 	close(fd); fd = -1;
-	if ((ret = lxc_netdev_rename_by_name(oldname, newname)) < 0) {
-		fprintf(stderr, "Error %d renaming netdev %s to %s in container\n", ret, oldname, newname);
+	if (!*newnamep) {
+		grab_newname = true;
+		*newnamep = VETH_DEF_NAME;
+		if (!(ifindex = if_nametoindex(oldname))) {
+			fprintf(stderr, "failed to get netdev index\n");
+			goto out_err;
+		}
+	}
+	if ((ret = lxc_netdev_rename_by_name(oldname, *newnamep)) < 0) {
+		fprintf(stderr, "Error %d renaming netdev %s to %s in container\n", ret, oldname, *newnamep);
 		goto out_err;
 	}
+	if (grab_newname) {
+		char ifname[IFNAMSIZ], *namep = ifname;
+		if (!if_indextoname(ifindex, namep)) {
+			fprintf(stderr, "Failed to get new netdev name\n");
+			goto out_err;
+		}
+		*newnamep = strdup(namep);
+		if (!*newnamep)
+			goto out_err;
+	}
 	if (setns(ofd, 0) < 0) {
 		fprintf(stderr, "Error returning to original netns\n");
 		close(ofd);
@@ -516,7 +540,7 @@ out_err:
 
 /*
  * If the caller (real uid, not effective uid) may read the
- * /proc/pid/net/ns, then it is either the caller's netns or one
+ * /proc/[pid]/ns/net, then it is either the caller's netns or one
  * which it created.
  */
 static bool may_access_netns(int pid)
@@ -562,7 +586,7 @@ int main(int argc, char *argv[])
 	char *me;
 	char *nicname = alloca(40);
 	char *cnic = NULL; // created nic name in container is returned here.
-	char *vethname;
+	char *vethname = NULL;
 	int pid;
 
 	if ((me = get_username()) == NULL) {
@@ -574,8 +598,6 @@ int main(int argc, char *argv[])
 		usage(argv[0], true);
 	if (argc >= 5)
 		vethname = argv[4];
-	else
-		vethname = "eth0";
 
 	errno = 0;
 	pid = (int) strtol(argv[1], NULL, 10);
@@ -610,7 +632,7 @@ int main(int argc, char *argv[])
 	}
 
 	// Now rename the link
-	if (rename_in_ns(pid, cnic, vethname) < 0) {
+	if (rename_in_ns(pid, cnic, &vethname) < 0) {
 		fprintf(stderr, "Failed to rename the link\n");
 		exit(1);
 	}
