@@ -37,6 +37,7 @@ char**
 convert_tuple_to_char_pointer_array(PyObject *argv) {
     int argc;
     int i, j;
+    char **result;
 
     /* not a list or tuple */
     if (!PyList_Check(argv) && !PyTuple_Check(argv)) {
@@ -46,7 +47,7 @@ convert_tuple_to_char_pointer_array(PyObject *argv) {
 
     argc = PySequence_Fast_GET_SIZE(argv);
 
-    char **result = (char**) calloc(argc + 1, sizeof(char*));
+    result = (char**) calloc(argc + 1, sizeof(char*));
 
     if (result == NULL) {
         PyErr_SetNone(PyExc_MemoryError);
@@ -54,11 +55,10 @@ convert_tuple_to_char_pointer_array(PyObject *argv) {
     }
 
     for (i = 0; i < argc; i++) {
-        PyObject *pyobj = PySequence_Fast_GET_ITEM(argv, i);
-        assert(pyobj != NULL);
-
         char *str = NULL;
         PyObject *pystr = NULL;
+        PyObject *pyobj = PySequence_Fast_GET_ITEM(argv, i);
+        assert(pyobj != NULL);
 
         if (!PyUnicode_Check(pyobj)) {
             PyErr_SetString(PyExc_ValueError, "Expected a string");
@@ -148,7 +148,7 @@ static lxc_attach_options_t *lxc_attach_parse_options(PyObject *kwds)
     PyObject *stdout_obj = NULL;
     PyObject *stderr_obj = NULL;
     PyObject *initial_cwd_obj = NULL;
-    PyObject *dummy;
+    PyObject *dummy = NULL;
     bool parse_result;
 
     lxc_attach_options_t default_options = LXC_ATTACH_OPTIONS_DEFAULT;
@@ -237,8 +237,7 @@ void lxc_attach_free_options(lxc_attach_options_t *options)
     int i;
     if (!options)
         return;
-    if (options->initial_cwd)
-        free(options->initial_cwd);
+    free(options->initial_cwd);
     if (options->extra_env_vars) {
         for (i = 0; options->extra_env_vars[i]; i++)
             free(options->extra_env_vars[i]);
@@ -257,7 +256,7 @@ static PyObject *
 LXC_arch_to_personality(PyObject *self, PyObject *arg)
 {
     long rv = -1;
-    PyObject *pystr;
+    PyObject *pystr = NULL;
     char *str;
 
     if (!PyUnicode_Check(arg)) {
@@ -329,12 +328,20 @@ LXC_get_global_config_item(PyObject *self, PyObject *args, PyObject *kwds)
 {
     static char *kwlist[] = {"key", NULL};
     char* key = NULL;
+    const char* value = NULL;
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "s|", kwlist,
                                       &key))
         return NULL;
 
-    return PyUnicode_FromString(lxc_get_global_config_item(key));
+    value = lxc_get_global_config_item(key);
+
+    if (!value) {
+        PyErr_SetString(PyExc_KeyError, "Invalid configuration key");
+        return NULL;
+    }
+
+    return PyUnicode_FromString(value);
 }
 
 static PyObject *
@@ -573,7 +580,9 @@ Container_attach_and_possibly_wait(Container *self, PyObject *args,
         goto out;
 
     if (wait) {
+        Py_BEGIN_ALLOW_THREADS
         ret = lxc_wait_for_pid_status(pid);
+        Py_END_ALLOW_THREADS
         /* handle case where attach fails */
         if (WIFEXITED(ret) && WEXITSTATUS(ret) == 255)
             ret = -1;
@@ -729,11 +738,12 @@ Container_create(Container *self, PyObject *args, PyObject *kwds)
     char* template_name = NULL;
     int flags = 0;
     char** create_args = {NULL};
-    PyObject *retval = NULL, *vargs = NULL;
+    PyObject *retval = NULL;
+    PyObject *vargs = NULL;
     int i = 0;
     static char *kwlist[] = {"template", "flags", "args", NULL};
 
-    if (! PyArg_ParseTupleAndKeywords(args, kwds, "s|iO", kwlist,
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "|siO", kwlist,
                                       &template_name, &flags, &vargs))
         return NULL;
 
@@ -795,6 +805,7 @@ Container_get_cgroup_item(Container *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"key", NULL};
     char* key = NULL;
     int len = 0;
+    char* value;
     PyObject *ret = NULL;
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "s", kwlist,
@@ -808,7 +819,7 @@ Container_get_cgroup_item(Container *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    char* value = (char*) malloc(sizeof(char)*len + 1);
+    value = (char*) malloc(sizeof(char)*len + 1);
     if (value == NULL)
         return PyErr_NoMemory();
 
@@ -830,6 +841,7 @@ Container_get_config_item(Container *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"key", NULL};
     char* key = NULL;
     int len = 0;
+    char* value;
     PyObject *ret = NULL;
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "s|", kwlist,
@@ -847,7 +859,7 @@ Container_get_config_item(Container *self, PyObject *args, PyObject *kwds)
         return PyUnicode_FromString("");
     }
 
-    char* value = (char*) malloc(sizeof(char)*len + 1);
+    value = (char*) malloc(sizeof(char)*len + 1);
     if (value == NULL)
         return PyErr_NoMemory();
 
@@ -876,6 +888,7 @@ Container_get_keys(Container *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"key", NULL};
     char* key = NULL;
     int len = 0;
+    char* value;
     PyObject *ret = NULL;
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "|s", kwlist,
@@ -889,7 +902,7 @@ Container_get_keys(Container *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    char* value = (char*) malloc(sizeof(char)*len + 1);
+    value = (char*) malloc(sizeof(char)*len + 1);
     if (value == NULL)
         return PyErr_NoMemory();
 
@@ -1221,7 +1234,7 @@ Container_snapshot(Container *self, PyObject *args, PyObject *kwds)
     int retval = 0;
     int ret = 0;
     char newname[20];
-    PyObject *py_comment_path;
+    PyObject *py_comment_path = NULL;
 
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "|O&", kwlist,
                                       PyUnicode_FSConverter, &py_comment_path))
